# coding: utf-8

"""
    allure-testops-service

    Branch **HEAD**  Commit **623f6ed302ba4b651cf9040faca4635af2d93b7c**

    The version of the OpenAPI document: 25.4.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictBool, StrictBytes, StrictInt, StrictStr
from typing import Any, List, Optional, Tuple, Union
from typing_extensions import Annotated
from src.client.generated.models.page_shared_step_attachment_row_dto import PageSharedStepAttachmentRowDto
from src.client.generated.models.shared_step_attachment_patch_dto import SharedStepAttachmentPatchDto
from src.client.generated.models.shared_step_attachment_row_dto import SharedStepAttachmentRowDto

from src.client.generated.api_client import ApiClient, RequestSerialized
from src.client.generated.api_response import ApiResponse
from src.client.generated.rest import RESTResponseType


class SharedStepAttachmentControllerApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client


    @validate_call
    async def create21(
        self,
        shared_step_id: StrictInt,
        file: List[Union[StrictBytes, StrictStr, Tuple[StrictStr, StrictBytes]]],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[SharedStepAttachmentRowDto]:
        """Upload new shared step attachments


        :param shared_step_id: (required)
        :type shared_step_id: int
        :param file: (required)
        :type file: List[bytearray]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create21_serialize(
            shared_step_id=shared_step_id,
            file=file,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[SharedStepAttachmentRowDto]",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    async def create21_with_http_info(
        self,
        shared_step_id: StrictInt,
        file: List[Union[StrictBytes, StrictStr, Tuple[StrictStr, StrictBytes]]],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[SharedStepAttachmentRowDto]]:
        """Upload new shared step attachments


        :param shared_step_id: (required)
        :type shared_step_id: int
        :param file: (required)
        :type file: List[bytearray]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create21_serialize(
            shared_step_id=shared_step_id,
            file=file,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[SharedStepAttachmentRowDto]",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    async def create21_without_preload_content(
        self,
        shared_step_id: StrictInt,
        file: List[Union[StrictBytes, StrictStr, Tuple[StrictStr, StrictBytes]]],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Upload new shared step attachments


        :param shared_step_id: (required)
        :type shared_step_id: int
        :param file: (required)
        :type file: List[bytearray]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._create21_serialize(
            shared_step_id=shared_step_id,
            file=file,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[SharedStepAttachmentRowDto]",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _create21_serialize(
        self,
        shared_step_id,
        file,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'file': 'csv',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if shared_step_id is not None:
            
            _query_params.append(('sharedStepId', shared_step_id))
            
        # process the header parameters
        # process the form parameters
        if file is not None:
            _files['file'] = file
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    '*/*'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'multipart/form-data'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/api/sharedstep/attachment',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    async def delete19(
        self,
        id: StrictInt,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> None:
        """Delete shared step attachment


        :param id: (required)
        :type id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete19_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    async def delete19_with_http_info(
        self,
        id: StrictInt,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[None]:
        """Delete shared step attachment


        :param id: (required)
        :type id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete19_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    async def delete19_without_preload_content(
        self,
        id: StrictInt,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Delete shared step attachment


        :param id: (required)
        :type id: int
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._delete19_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': None,
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _delete19_serialize(
        self,
        id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter




        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='DELETE',
            resource_path='/api/sharedstep/attachment/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    async def find_all17(
        self,
        shared_step_id: StrictInt,
        page: Annotated[Optional[Annotated[int, Field(strict=True, ge=0)]], Field(description="Zero-based page index (0..N)")] = None,
        size: Annotated[Optional[Annotated[int, Field(strict=True, ge=1)]], Field(description="The size of the page to be returned")] = None,
        sort: Annotated[Optional[List[StrictStr]], Field(description="Sorting criteria in the format: property(,asc|desc). Default sort order is ascending. Multiple sort criteria are supported.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> PageSharedStepAttachmentRowDto:
        """Find attachments for shared step


        :param shared_step_id: (required)
        :type shared_step_id: int
        :param page: Zero-based page index (0..N)
        :type page: int
        :param size: The size of the page to be returned
        :type size: int
        :param sort: Sorting criteria in the format: property(,asc|desc). Default sort order is ascending. Multiple sort criteria are supported.
        :type sort: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._find_all17_serialize(
            shared_step_id=shared_step_id,
            page=page,
            size=size,
            sort=sort,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PageSharedStepAttachmentRowDto",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    async def find_all17_with_http_info(
        self,
        shared_step_id: StrictInt,
        page: Annotated[Optional[Annotated[int, Field(strict=True, ge=0)]], Field(description="Zero-based page index (0..N)")] = None,
        size: Annotated[Optional[Annotated[int, Field(strict=True, ge=1)]], Field(description="The size of the page to be returned")] = None,
        sort: Annotated[Optional[List[StrictStr]], Field(description="Sorting criteria in the format: property(,asc|desc). Default sort order is ascending. Multiple sort criteria are supported.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[PageSharedStepAttachmentRowDto]:
        """Find attachments for shared step


        :param shared_step_id: (required)
        :type shared_step_id: int
        :param page: Zero-based page index (0..N)
        :type page: int
        :param size: The size of the page to be returned
        :type size: int
        :param sort: Sorting criteria in the format: property(,asc|desc). Default sort order is ascending. Multiple sort criteria are supported.
        :type sort: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._find_all17_serialize(
            shared_step_id=shared_step_id,
            page=page,
            size=size,
            sort=sort,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PageSharedStepAttachmentRowDto",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    async def find_all17_without_preload_content(
        self,
        shared_step_id: StrictInt,
        page: Annotated[Optional[Annotated[int, Field(strict=True, ge=0)]], Field(description="Zero-based page index (0..N)")] = None,
        size: Annotated[Optional[Annotated[int, Field(strict=True, ge=1)]], Field(description="The size of the page to be returned")] = None,
        sort: Annotated[Optional[List[StrictStr]], Field(description="Sorting criteria in the format: property(,asc|desc). Default sort order is ascending. Multiple sort criteria are supported.")] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Find attachments for shared step


        :param shared_step_id: (required)
        :type shared_step_id: int
        :param page: Zero-based page index (0..N)
        :type page: int
        :param size: The size of the page to be returned
        :type size: int
        :param sort: Sorting criteria in the format: property(,asc|desc). Default sort order is ascending. Multiple sort criteria are supported.
        :type sort: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._find_all17_serialize(
            shared_step_id=shared_step_id,
            page=page,
            size=size,
            sort=sort,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "PageSharedStepAttachmentRowDto",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _find_all17_serialize(
        self,
        shared_step_id,
        page,
        size,
        sort,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'sort': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if shared_step_id is not None:
            
            _query_params.append(('sharedStepId', shared_step_id))
            
        if page is not None:
            
            _query_params.append(('page', page))
            
        if size is not None:
            
            _query_params.append(('size', size))
            
        if sort is not None:
            
            _query_params.append(('sort', sort))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    '*/*'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/sharedstep/attachment',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    async def patch19(
        self,
        id: StrictInt,
        shared_step_attachment_patch_dto: SharedStepAttachmentPatchDto,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SharedStepAttachmentRowDto:
        """Patch shared step attachment


        :param id: (required)
        :type id: int
        :param shared_step_attachment_patch_dto: (required)
        :type shared_step_attachment_patch_dto: SharedStepAttachmentPatchDto
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._patch19_serialize(
            id=id,
            shared_step_attachment_patch_dto=shared_step_attachment_patch_dto,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SharedStepAttachmentRowDto",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    async def patch19_with_http_info(
        self,
        id: StrictInt,
        shared_step_attachment_patch_dto: SharedStepAttachmentPatchDto,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SharedStepAttachmentRowDto]:
        """Patch shared step attachment


        :param id: (required)
        :type id: int
        :param shared_step_attachment_patch_dto: (required)
        :type shared_step_attachment_patch_dto: SharedStepAttachmentPatchDto
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._patch19_serialize(
            id=id,
            shared_step_attachment_patch_dto=shared_step_attachment_patch_dto,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SharedStepAttachmentRowDto",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    async def patch19_without_preload_content(
        self,
        id: StrictInt,
        shared_step_attachment_patch_dto: SharedStepAttachmentPatchDto,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Patch shared step attachment


        :param id: (required)
        :type id: int
        :param shared_step_attachment_patch_dto: (required)
        :type shared_step_attachment_patch_dto: SharedStepAttachmentPatchDto
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._patch19_serialize(
            id=id,
            shared_step_attachment_patch_dto=shared_step_attachment_patch_dto,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SharedStepAttachmentRowDto",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _patch19_serialize(
        self,
        id,
        shared_step_attachment_patch_dto,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if shared_step_attachment_patch_dto is not None:
            _body_params = shared_step_attachment_patch_dto


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    '*/*'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'application/json'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='PATCH',
            resource_path='/api/sharedstep/attachment/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    async def read_content3(
        self,
        id: StrictInt,
        inline: Optional[StrictBool] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> object:
        """Get attachment content by id


        :param id: (required)
        :type id: int
        :param inline:
        :type inline: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._read_content3_serialize(
            id=id,
            inline=inline,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "object",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    async def read_content3_with_http_info(
        self,
        id: StrictInt,
        inline: Optional[StrictBool] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[object]:
        """Get attachment content by id


        :param id: (required)
        :type id: int
        :param inline:
        :type inline: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._read_content3_serialize(
            id=id,
            inline=inline,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "object",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    async def read_content3_without_preload_content(
        self,
        id: StrictInt,
        inline: Optional[StrictBool] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get attachment content by id


        :param id: (required)
        :type id: int
        :param inline:
        :type inline: bool
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._read_content3_serialize(
            id=id,
            inline=inline,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "object",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _read_content3_serialize(
        self,
        id,
        inline,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        if inline is not None:
            
            _query_params.append(('inline', inline))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    '*/*'
                ]
            )


        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='GET',
            resource_path='/api/sharedstep/attachment/{id}/content',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )




    @validate_call
    async def update_content3(
        self,
        id: StrictInt,
        file: Union[StrictBytes, StrictStr, Tuple[StrictStr, StrictBytes]],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SharedStepAttachmentRowDto:
        """Update shared step attachment content


        :param id: (required)
        :type id: int
        :param file: (required)
        :type file: bytearray
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_content3_serialize(
            id=id,
            file=file,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SharedStepAttachmentRowDto",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    async def update_content3_with_http_info(
        self,
        id: StrictInt,
        file: Union[StrictBytes, StrictStr, Tuple[StrictStr, StrictBytes]],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SharedStepAttachmentRowDto]:
        """Update shared step attachment content


        :param id: (required)
        :type id: int
        :param file: (required)
        :type file: bytearray
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_content3_serialize(
            id=id,
            file=file,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SharedStepAttachmentRowDto",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    async def update_content3_without_preload_content(
        self,
        id: StrictInt,
        file: Union[StrictBytes, StrictStr, Tuple[StrictStr, StrictBytes]],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)]
            ]
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Update shared step attachment content


        :param id: (required)
        :type id: int
        :param file: (required)
        :type file: bytearray
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._update_content3_serialize(
            id=id,
            file=file,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SharedStepAttachmentRowDto",
        }
        response_data = await self.api_client.call_api(
            *_param,
            _request_timeout=_request_timeout
        )
        return response_data.response


    def _update_content3_serialize(
        self,
        id,
        file,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        if file is not None:
            _files['file'] = file
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    '*/*'
                ]
            )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = (
                self.api_client.select_header_content_type(
                    [
                        'multipart/form-data'
                    ]
                )
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = [
        ]

        return self.api_client.param_serialize(
            method='PUT',
            resource_path='/api/sharedstep/attachment/{id}/content',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth
        )





# coding: utf-8

"""
    allure-testops-service

    Branch **HEAD**  Commit **623f6ed302ba4b651cf9040faca4635af2d93b7c**

    The version of the OpenAPI document: 25.4.1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from src.client.generated.models.environment_set_dto import EnvironmentSetDto
from src.client.generated.models.external_link_dto import ExternalLinkDto
from src.client.generated.models.issue_dto import IssueDto
from src.client.generated.models.job_parameter_dto import JobParameterDto
from src.client.generated.models.launch_tag_dto import LaunchTagDto
from src.client.generated.models.tree_selection_dto import TreeSelectionDto
from typing import Optional, Set
from typing_extensions import Self

class JobRunRequestDto(BaseModel):
    """
    JobRunRequestDto
    """ # noqa: E501
    env_var_value_sets: Optional[List[EnvironmentSetDto]] = Field(default=None, alias="envVarValueSets")
    issues: Optional[List[IssueDto]] = None
    launch_name: Annotated[str, Field(min_length=1, strict=True, max_length=255)] = Field(alias="launchName")
    links: Optional[List[ExternalLinkDto]] = None
    parameters: Optional[List[JobParameterDto]] = None
    selection: Optional[TreeSelectionDto] = None
    tags: Optional[List[LaunchTagDto]] = None
    __properties: ClassVar[List[str]] = ["envVarValueSets", "issues", "launchName", "links", "parameters", "selection", "tags"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of JobRunRequestDto from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in env_var_value_sets (list)
        _items = []
        if self.env_var_value_sets:
            for _item_env_var_value_sets in self.env_var_value_sets:
                if _item_env_var_value_sets:
                    _items.append(_item_env_var_value_sets.to_dict())
            _dict['envVarValueSets'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in issues (list)
        _items = []
        if self.issues:
            for _item_issues in self.issues:
                if _item_issues:
                    _items.append(_item_issues.to_dict())
            _dict['issues'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in links (list)
        _items = []
        if self.links:
            for _item_links in self.links:
                if _item_links:
                    _items.append(_item_links.to_dict())
            _dict['links'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in parameters (list)
        _items = []
        if self.parameters:
            for _item_parameters in self.parameters:
                if _item_parameters:
                    _items.append(_item_parameters.to_dict())
            _dict['parameters'] = _items
        # override the default output from pydantic by calling `to_dict()` of selection
        if self.selection:
            _dict['selection'] = self.selection.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in tags (list)
        _items = []
        if self.tags:
            for _item_tags in self.tags:
                if _item_tags:
                    _items.append(_item_tags.to_dict())
            _dict['tags'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of JobRunRequestDto from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "envVarValueSets": [EnvironmentSetDto.from_dict(_item) for _item in obj["envVarValueSets"]] if obj.get("envVarValueSets") is not None else None,
            "issues": [IssueDto.from_dict(_item) for _item in obj["issues"]] if obj.get("issues") is not None else None,
            "launchName": obj.get("launchName"),
            "links": [ExternalLinkDto.from_dict(_item) for _item in obj["links"]] if obj.get("links") is not None else None,
            "parameters": [JobParameterDto.from_dict(_item) for _item in obj["parameters"]] if obj.get("parameters") is not None else None,
            "selection": TreeSelectionDto.from_dict(obj["selection"]) if obj.get("selection") is not None else None,
            "tags": [LaunchTagDto.from_dict(_item) for _item in obj["tags"]] if obj.get("tags") is not None else None
        })
        return _obj





"""E2E server entrypoint that routes telemetry to the test collector."""

from __future__ import annotations

import importlib
import os
from dataclasses import replace

from src.utils import config as config_module


def _configure_test_telemetry() -> None:
    base_url = os.getenv("LUCIUS_E2E_UMAMI_BASE_URL")
    if not base_url:
        return
    config_module.telemetry_config = replace(config_module.telemetry_config, umami_base_url=base_url)


_configure_test_telemetry()


if __name__ == "__main__":
    start = importlib.import_module("src.main").start
    start()



from unittest.mock import AsyncMock, MagicMock

import pytest

from src.client import AllureClient, PageTestCaseDto, TestCaseDto, TestCaseDtoWithCF, TestCaseScenarioV2Dto
from src.client.exceptions import AllureNotFoundError, AllureValidationError, TestCaseNotFoundError
from src.client.generated.models.custom_field_dto import CustomFieldDto
from src.client.generated.models.custom_field_value_with_cf_dto import CustomFieldValueWithCfDto
from src.client.generated.models.test_tag_dto import TestTagDto
from src.services.search_service import SearchQueryParser, SearchService, TestCaseDetails
from src.tools.search import _format_search_results, _format_test_case_details


@pytest.fixture
def mock_client() -> AllureClient:
    client = MagicMock(spec=AllureClient)
    client.list_test_cases = AsyncMock()
    client.get_test_case = AsyncMock()
    client.get_test_case_scenario = AsyncMock()
    client.get_project.return_value = 123
    return client


@pytest.fixture
def service(mock_client: AllureClient) -> SearchService:
    return SearchService(client=mock_client)


@pytest.mark.asyncio
async def test_get_test_case_details_returns_case_and_scenario(
    service: SearchService, mock_client: AllureClient
) -> None:
    test_case = TestCaseDto(id=123, name="Login", description="d")
    scenario = TestCaseScenarioV2Dto(steps=[])
    mock_client.get_test_case = AsyncMock(return_value=test_case)
    mock_client.get_test_case_scenario = AsyncMock(return_value=scenario)

    details = await service.get_test_case_details(123)

    assert isinstance(details, TestCaseDetails)
    assert details.test_case.id == 123
    assert details.scenario is scenario
    mock_client.get_test_case.assert_awaited_once_with(123)
    mock_client.get_test_case_scenario.assert_awaited_once_with(123)


@pytest.mark.asyncio
async def test_get_test_case_details_maps_not_found(service: SearchService, mock_client: AllureClient) -> None:
    mock_client.get_test_case = AsyncMock(side_effect=AllureNotFoundError("nf"))
    mock_client.get_test_case_scenario = AsyncMock()

    with pytest.raises(TestCaseNotFoundError):
        await service.get_test_case_details(999)

    mock_client.get_test_case.assert_awaited_once_with(999)
    mock_client.get_test_case_scenario.assert_not_awaited()


@pytest.mark.asyncio
async def test_get_test_case_details_validates_id(service: SearchService) -> None:
    with pytest.raises(AllureValidationError, match="Test case ID must be a positive integer"):
        await service.get_test_case_details(0)


def test_format_test_case_details_handles_fields_and_steps() -> None:
    tc = TestCaseDtoWithCF(
        id=1,
        name="Login",
        description="Desc",
        precondition="Pre",
        tags=[],
    )
    # Simulate nested steps using simple objects to bypass generated validation
    from types import SimpleNamespace

    class SimpleStep(SimpleNamespace):
        def __init__(self, body: str, expected: str | None = None, steps: list | None = None) -> None:
            super().__init__(body=body, expected=expected, steps=steps or [])

    child = SimpleStep("Sub-step", "Outcome")
    parent = SimpleStep("Do X", "See Y", steps=[child])

    scenario = SimpleNamespace(steps=[parent], attachments=[SimpleNamespace(id=10, name="log.txt")])
    tc.custom_fields = [CustomFieldValueWithCfDto(custom_field=CustomFieldDto(name="Layer"), name="UI")]
    details = TestCaseDetails(test_case=tc, scenario=scenario)

    text = _format_test_case_details(details)

    assert "Test Case #1: Login" in text
    assert "Description" in text
    assert "Preconditions" in text
    assert "1. Do X → See Y" in text
    assert "1.1. Sub-step → Outcome" in text
    assert "Custom Fields" in text
    assert "Layer=UI" in text
    assert "Attachments" in text
    assert "log.txt" in text
    assert "id: 10" in text


@pytest.mark.asyncio
async def test_list_test_cases_returns_paginated_results(service: SearchService, mock_client: AllureClient) -> None:
    page = PageTestCaseDto(
        content=[TestCaseDto(id=1, name="Login Flow")],
        total_elements=1,
        number=0,
        size=20,
        total_pages=1,
    )
    mock_client.list_test_cases.return_value = page

    result = await service.list_test_cases()

    assert result.total == 1
    assert result.page == 0
    assert result.total_pages == 1
    assert result.items[0].id == 1
    assert result.items[0].name == "Login Flow"
    mock_client.list_test_cases.assert_called_once_with(
        project_id=123,
        page=0,
        size=20,
        search=None,
        tags=None,
        status=None,
    )


@pytest.mark.asyncio
async def test_list_test_cases_passes_filters(service: SearchService, mock_client: AllureClient) -> None:
    page = PageTestCaseDto(content=[], total_elements=0, number=0, size=20, total_pages=0)
    mock_client.list_test_cases.return_value = page

    await service.list_test_cases(
        page=1,
        size=50,
        name_filter="login",
        tags=["smoke", "auth"],
        status="Draft",
    )

    mock_client.list_test_cases.assert_called_once_with(
        project_id=123,
        page=1,
        size=50,
        search="login",
        tags=["smoke", "auth"],
        status="Draft",
    )


@pytest.mark.asyncio
async def test_search_test_cases_parses_query(service: SearchService, mock_client: AllureClient) -> None:
    page = PageTestCaseDto(content=[], total_elements=0, number=0, size=20, total_pages=0)
    mock_client.list_test_cases.return_value = page

    await service.search_test_cases(
        query="login tag:SMOKE tag:auth",
        page=2,
        size=10,
    )

    mock_client.list_test_cases.assert_called_once_with(
        project_id=123,
        page=2,
        size=10,
        search="login",
        tags=["SMOKE", "auth"],
        status=None,
    )
    assert SearchService.search_test_cases.__doc__


@pytest.mark.asyncio
async def test_list_test_cases_validates_project_id(service: SearchService) -> None:
    service._project_id = 0
    with pytest.raises(AllureValidationError, match="Project ID is required and must be positive"):
        await service.list_test_cases()


@pytest.mark.asyncio
async def test_list_test_cases_validates_pagination(service: SearchService) -> None:
    with pytest.raises(AllureValidationError, match="Page must be a non-negative integer"):
        await service.list_test_cases(page=-1)

    with pytest.raises(AllureValidationError, match="Size must be a positive integer"):
        await service.list_test_cases(size=0)

    with pytest.raises(AllureValidationError, match="Size must be 100 or less"):
        await service.list_test_cases(size=101)


def test_format_search_results_handles_empty() -> None:
    empty_page = PageTestCaseDto(content=[], total_elements=0, number=0, size=20, total_pages=0)
    mock_client = MagicMock(spec=AllureClient)
    mock_client.get_project.return_value = 1
    empty_result = SearchService(client=mock_client)._build_result(empty_page)

    text = _format_search_results(empty_result, "login tag:auth")

    assert text == "No test cases found matching 'login tag:auth'."


def test_search_query_parser_parses_name_only() -> None:
    parsed = SearchQueryParser.parse("login flow")

    assert parsed.name_query == "login flow"
    assert parsed.tags == []


def test_search_query_parser_parses_single_tag() -> None:
    parsed = SearchQueryParser.parse("tag:smoke")

    assert parsed.name_query is None
    assert parsed.tags == ["smoke"]


def test_search_query_parser_parses_multiple_tags() -> None:
    parsed = SearchQueryParser.parse("tag:smoke tag:auth")

    assert parsed.name_query is None
    assert parsed.tags == ["smoke", "auth"]


def test_search_query_parser_parses_combined() -> None:
    parsed = SearchQueryParser.parse("login tag:auth")

    assert parsed.name_query == "login"
    assert parsed.tags == ["auth"]


def test_search_query_parser_normalizes_case() -> None:
    parsed = SearchQueryParser.parse("tag:SMOKE tag:Auth")

    assert parsed.tags == ["SMOKE", "Auth"]


def test_format_search_results_includes_tags_and_pagination() -> None:
    tag = TestTagDto.model_construct(name="smoke")
    items = [
        TestCaseDto(id=1, name="Login Flow", tags=[tag]),
        TestCaseDto(id=2, name="Logout", tags=[]),
    ]
    page = PageTestCaseDto(content=items, total_elements=2, number=0, size=1, total_pages=2)
    mock_client = MagicMock(spec=AllureClient)
    mock_client.get_project.return_value = 1
    result = SearchService(client=mock_client)._build_result(page)

    text = _format_search_results(result, "login")

    assert "Found 2 test cases matching 'login':" in text
    assert "[#1] Login Flow" in text
    assert "tags: smoke" in text
    assert "[#2] Logout" in text
    assert "tags: none" in text
    assert "Showing page 1 of 2" in text


# =============================================
# AQL Search Tests
# =============================================


@pytest.fixture
def mock_client_with_aql() -> AllureClient:
    """Client mock with AQL-specific methods."""
    client = MagicMock(spec=AllureClient)
    client.list_test_cases = AsyncMock()
    client.get_test_case = AsyncMock()
    client.get_test_case_scenario = AsyncMock()
    client.search_test_cases_aql = AsyncMock()
    client.validate_test_case_query = AsyncMock()
    client.get_project.return_value = 123
    return client


@pytest.fixture
def aql_service(mock_client_with_aql: AllureClient) -> SearchService:
    return SearchService(client=mock_client_with_aql)


@pytest.mark.asyncio
async def test_search_test_cases_aql_bypasses_parser(
    aql_service: SearchService, mock_client_with_aql: AllureClient
) -> None:
    """When AQL is provided, SearchService should bypass query parser and call AQL client method."""
    page = PageTestCaseDto(
        content=[TestCaseDto(id=1, name="Failed Test")],
        total_elements=1,
        number=0,
        size=20,
        total_pages=1,
    )
    mock_client_with_aql.validate_test_case_query.return_value = (True, 1)
    mock_client_with_aql.search_test_cases_aql.return_value = page

    result = await aql_service.search_test_cases(
        aql='status="failed" and tag="regression"',
        page=0,
        size=20,
    )

    assert result.total == 1
    assert result.items[0].name == "Failed Test"
    mock_client_with_aql.validate_test_case_query.assert_awaited_once_with(
        project_id=123,
        rql='status="failed" and tag="regression"',
    )
    mock_client_with_aql.search_test_cases_aql.assert_awaited_once_with(
        project_id=123,
        rql='status="failed" and tag="regression"',
        page=0,
        size=20,
    )
    # Should NOT call list_test_cases when using AQL
    mock_client_with_aql.list_test_cases.assert_not_awaited()


@pytest.mark.asyncio
async def test_search_test_cases_aql_validates_before_search(
    aql_service: SearchService, mock_client_with_aql: AllureClient
) -> None:
    """AQL search should validate syntax before executing query."""
    mock_client_with_aql.validate_test_case_query.return_value = (False, None)

    with pytest.raises(AllureValidationError, match="Invalid AQL syntax"):
        await aql_service.search_test_cases(
            aql="invalid syntax here",
            page=0,
            size=20,
        )

    mock_client_with_aql.validate_test_case_query.assert_awaited_once()
    # Should NOT proceed to search after validation failure
    mock_client_with_aql.search_test_cases_aql.assert_not_awaited()


@pytest.mark.asyncio
async def test_search_test_cases_aql_empty_string_raises(aql_service: SearchService) -> None:
    """Empty AQL query should raise validation error."""
    with pytest.raises(AllureValidationError, match="AQL query must be a non-empty string"):
        await aql_service.search_test_cases(aql="   ", page=0, size=20)


@pytest.mark.asyncio
async def test_search_test_cases_simple_query_still_works(
    aql_service: SearchService, mock_client_with_aql: AllureClient
) -> None:
    """Simple query mode should work as before when AQL is not provided."""
    page = PageTestCaseDto(content=[], total_elements=0, number=0, size=20, total_pages=0)
    mock_client_with_aql.list_test_cases.return_value = page

    await aql_service.search_test_cases(query="login tag:smoke", page=0, size=20)

    # Should use simple query path, not AQL
    mock_client_with_aql.list_test_cases.assert_called_once_with(
        project_id=123,
        page=0,
        size=20,
        search="login",
        tags=["smoke"],
        status=None,
    )
    mock_client_with_aql.search_test_cases_aql.assert_not_awaited()
    mock_client_with_aql.validate_test_case_query.assert_not_awaited()


@pytest.mark.asyncio
async def test_search_test_cases_requires_query_or_aql(aql_service: SearchService) -> None:
    """Must provide either query or aql parameter."""
    with pytest.raises(AllureValidationError, match="Search query must be a non-empty string"):
        await aql_service.search_test_cases(page=0, size=20)


@pytest.mark.asyncio
async def test_search_test_cases_aql_respects_pagination(
    aql_service: SearchService, mock_client_with_aql: AllureClient
) -> None:
    """AQL search should respect pagination parameters."""
    page = PageTestCaseDto(
        content=[TestCaseDto(id=1, name="Test")],
        total_elements=50,
        number=2,
        size=10,
        total_pages=5,
    )
    mock_client_with_aql.validate_test_case_query.return_value = (True, 50)
    mock_client_with_aql.search_test_cases_aql.return_value = page

    result = await aql_service.search_test_cases(
        aql='tag in ["smoke", "e2e"]',
        page=2,
        size=10,
    )

    assert result.page == 2
    assert result.size == 10
    assert result.total_pages == 5
    mock_client_with_aql.search_test_cases_aql.assert_awaited_once_with(
        project_id=123,
        rql='tag in ["smoke", "e2e"]',
        page=2,
        size=10,
    )


def test_format_search_results_with_aql_query() -> None:
    """Search results formatter should work with AQL query strings."""
    items = [
        TestCaseDto(id=1, name="Failed Test", tags=[TestTagDto.model_construct(name="regression")]),
    ]
    page = PageTestCaseDto(content=items, total_elements=1, number=0, size=20, total_pages=1)
    mock_client = MagicMock(spec=AllureClient)
    mock_client.get_project.return_value = 1
    result = SearchService(client=mock_client)._build_result(page)

    text = _format_search_results(result, 'status="failed" and tag="regression"')

    assert 'Found 1 test cases matching \'status="failed" and tag="regression"\'' in text
    assert "[#1] Failed Test" in text
    assert "tags: regression" in text



"""E2E tests for custom field value CRUD operations."""

import asyncio
import uuid

import pytest

from src.client import AllureClient
from src.services.custom_field_value_service import CustomFieldValueService
from src.services.test_case_service import TestCaseService


@pytest.mark.asyncio
async def test_custom_field_values_crud(allure_client: AllureClient) -> None:
    service = CustomFieldValueService(allure_client)
    test_case_service = TestCaseService(allure_client)
    project_id = allure_client.get_project()

    resolved = await test_case_service._get_resolved_custom_fields(project_id)
    optional_fields = [
        name
        for name, info in resolved.items()
        if not info["required"] and isinstance(info["project_cf_id"], int) and info["project_cf_id"] > 0
    ]
    if not optional_fields:
        pytest.skip("No optional custom fields with positive IDs available in this project.")

    field_name = optional_fields[0]

    unique_suffix = uuid.uuid4().hex[:8]
    value_name = f"e2e-value-{unique_suffix}"
    updated_name = f"e2e-updated-{unique_suffix}"
    temp_value_names = {value_name, updated_name}

    try:
        created = await service.create_custom_field_value(custom_field_name=field_name, name=value_name)
        assert created.id is not None

        deleted = await service.delete_custom_field_value(custom_field_name=field_name, cfv_id=created.id)
        assert deleted is True

        deleted_again = await service.delete_custom_field_value(custom_field_name=field_name, cfv_id=created.id)
        assert deleted_again is False

        created = await service.create_custom_field_value(custom_field_name=field_name, name=value_name)
        assert created.id is not None

        page = await service.list_custom_field_values(
            custom_field_name=field_name,
            size=100,
            sort=["id,desc"],
        )
        values = page.content or []
        assert any(v.id == created.id for v in values)

        # IDs are changed after update
        await service.update_custom_field_value(custom_field_name=field_name, cfv_id=created.id, name=updated_name)

        updated_values = []
        for _ in range(10):
            page_after_update = await service.list_custom_field_values(
                custom_field_name=field_name,
                size=100,
                sort=["id,desc"],
            )
            updated_values = page_after_update.content or []
            # IDs are changed after update
            if any(v.name == updated_name for v in updated_values):
                break
            await asyncio.sleep(0.5)

        if not any(v.name == updated_name for v in updated_values):
            pytest.skip("Custom field value rename not reflected in list output.")

        assert any(v.name == updated_name for v in updated_values)
    finally:
        for temp_name in temp_value_names:
            await service.delete_custom_field_value(
                custom_field_name=field_name,
                cfv_name=temp_name,
                force=True,
            )



"""E2E tests for test hierarchy management workflows."""

import asyncio
import re
from uuid import uuid4

from src.client import AllureClient
from src.client.generated.models.test_case_tree_leaf_dto_v2 import TestCaseTreeLeafDtoV2
from src.services.test_hierarchy_service import TestHierarchyService
from src.tools.assign_test_cases_to_suite import assign_test_cases_to_suite
from src.tools.create_test_case import create_test_case
from src.tools.create_test_suite import create_test_suite
from src.tools.delete_test_case import delete_test_case
from src.tools.delete_test_suite import delete_test_suite
from src.tools.list_test_suites import list_test_suites
from tests.e2e.helpers.cleanup import CleanupTracker


def _collect_suite_ids(nodes: list) -> list[int]:
    ids: list[int] = []
    for node in nodes:
        ids.append(node.id)
        ids.extend(_collect_suite_ids(node.children))
    return ids


async def _wait_for_suite_absence(
    service: TestHierarchyService,
    suite_id: int,
    *,
    retries: int = 120,
    delay_seconds: float = 1.0,
) -> bool:
    for _ in range(retries):
        _tree, suites = await service.list_test_suites(include_empty=True)
        if suite_id not in _collect_suite_ids(suites):
            return True
        await asyncio.sleep(delay_seconds)
    return False


async def test_e2e_hierarchy_create_and_list_suites(
    project_id: int,
    allure_client: AllureClient,
    cleanup_tracker: CleanupTracker,
) -> None:
    """Create root and nested suites, then verify they appear in hierarchy list."""
    service = TestHierarchyService(allure_client)

    run_id = uuid4().hex[:8]
    root_name = f"E2E-Hierarchy-Root-{project_id}-{run_id}"
    nested_name = f"E2E-Hierarchy-Nested-{project_id}-{run_id}"
    cleanup_tracker.track_custom_field_value_name(root_name)
    cleanup_tracker.track_custom_field_value_name(nested_name)

    root_suite = await service.create_test_suite(name=root_name)
    assert root_suite.id is not None
    cleanup_tracker.track_test_suite(root_suite.id)

    nested_suite = await service.create_test_suite(
        name=nested_name,
        parent_suite_id=root_suite.id,
    )
    assert nested_suite.id is not None
    cleanup_tracker.track_test_suite(nested_suite.id)

    _tree, suites = await service.list_test_suites(include_empty=True)
    assert isinstance(suites, list)


async def test_e2e_hierarchy_assign_test_cases_to_suite(
    project_id: int,
    allure_client: AllureClient,
    cleanup_tracker: CleanupTracker,
) -> None:
    """Create test case and assign it into created suite."""
    service = TestHierarchyService(allure_client)

    run_id = uuid4().hex[:8]
    suite_name = f"E2E-Assign-Suite-{project_id}-{run_id}"
    cleanup_tracker.track_custom_field_value_name(suite_name)
    suite = await service.create_test_suite(name=suite_name)
    assert suite.id is not None
    cleanup_tracker.track_test_suite(suite.id)

    create_output = await create_test_case(
        name=f"E2E Hierarchy Assignment Case {run_id}",
        project_id=project_id,
    )
    assert "Created Test Case ID:" in create_output

    match = re.search(r"ID: (\d+)", create_output)
    assert match is not None
    test_case_id = int(match.group(1))
    cleanup_tracker.track_test_case(test_case_id)

    assigned_count = await service.assign_test_cases_to_suite(
        suite_id=suite.id,
        test_case_ids=[test_case_id],
    )

    assert assigned_count == 1

    tree = await service._resolve_tree(None)
    tree_id = tree.id
    assert tree_id is not None

    suite_node = await allure_client.get_tree_node(
        project_id=project_id,
        tree_id=tree_id,
        parent_node_id=suite.id,
        page=0,
        size=500,
    )
    assert suite_node.children is not None
    suite_children = suite_node.children.content or []
    leaf_ids = [
        item.actual_instance.test_case_id
        for item in suite_children
        if isinstance(item.actual_instance, TestCaseTreeLeafDtoV2) and item.actual_instance.test_case_id is not None
    ]
    assert test_case_id in leaf_ids

    delete_output = await delete_test_case(test_case_id=test_case_id, confirm=True, project_id=project_id)
    assert "Archived Test Case" in delete_output or "already archived" in delete_output


async def test_e2e_hierarchy_tools_smoke(
    project_id: int,
    cleanup_tracker: CleanupTracker,
) -> None:
    """Run hierarchy tools in real environment and validate outputs."""
    run_id = uuid4().hex[:8]

    suite_name = f"E2E-Tool-Suite-{project_id}-{run_id}"
    cleanup_tracker.track_custom_field_value_name(suite_name)
    create_output = await create_test_suite(name=suite_name, project_id=project_id)
    assert "✅ Test suite created successfully!" in create_output

    match = re.search(r"ID: (\d+)", create_output)
    assert match is not None
    suite_id = int(match.group(1))
    cleanup_tracker.track_test_suite(suite_id)

    list_output = await list_test_suites(project_id=project_id)
    assert "Tree:" in list_output

    tc_output = await create_test_case(name=f"E2E Tool Assign Case {run_id}", project_id=project_id)
    tc_match = re.search(r"ID: (\d+)", tc_output)
    assert tc_match is not None
    tc_id = int(tc_match.group(1))
    cleanup_tracker.track_test_case(tc_id)

    assign_output = await assign_test_cases_to_suite(
        suite_id=suite_id,
        test_case_ids=[tc_id],
        project_id=project_id,
    )
    assert "Assigned" in assign_output

    delete_output = await delete_test_case(test_case_id=tc_id, confirm=True, project_id=project_id)
    assert "Archived Test Case" in delete_output or "already archived" in delete_output


async def test_e2e_hierarchy_delete_suite_lifecycle(
    project_id: int,
    allure_client: AllureClient,
    cleanup_tracker: CleanupTracker,
) -> None:
    """Create suite + case, assign case, delete suite, then verify suite is removed."""
    service = TestHierarchyService(allure_client)
    run_id = uuid4().hex[:8]

    root_name = f"E2E-Delete-Root-{project_id}-{run_id}"
    nested_name = f"E2E-Delete-Nested-{project_id}-{run_id}"
    cleanup_tracker.track_custom_field_value_name(root_name)
    cleanup_tracker.track_custom_field_value_name(nested_name)

    root_suite = await service.create_test_suite(name=root_name)
    assert root_suite.id is not None
    cleanup_tracker.track_test_suite(root_suite.id)

    nested_suite = await service.create_test_suite(name=nested_name, parent_suite_id=root_suite.id)
    assert nested_suite.id is not None
    cleanup_tracker.track_test_suite(nested_suite.id)

    tc_output = await create_test_case(name=f"E2E Delete Suite Case {run_id}", project_id=project_id)
    match = re.search(r"ID: (\d+)", tc_output)
    assert match is not None
    test_case_id = int(match.group(1))
    cleanup_tracker.track_test_case(test_case_id)

    assign_output = await assign_test_cases_to_suite(
        suite_id=nested_suite.id,
        test_case_ids=[test_case_id],
        project_id=project_id,
    )
    assert "Assigned 1 test case(s)" in assign_output

    delete_output = await delete_test_suite(
        suite_id=nested_suite.id,
        confirm=True,
        project_id=project_id,
    )
    assert f"✅ Test suite {nested_suite.id} deleted successfully (idempotent)." == delete_output

    second_delete_output = await delete_test_suite(
        suite_id=nested_suite.id,
        confirm=True,
        project_id=project_id,
    )
    assert second_delete_output == delete_output

    # In some TestOps setups, suites with assigned leaves become removable
    # only after leaf cleanup. Keep the lifecycle deterministic for both modes.
    delete_case_output = await delete_test_case(
        test_case_id=test_case_id,
        confirm=True,
        project_id=project_id,
    )
    assert "Archived Test Case" in delete_case_output or "already archived" in delete_case_output

    third_delete_output = await delete_test_suite(
        suite_id=nested_suite.id,
        confirm=True,
        project_id=project_id,
    )
    assert third_delete_output == delete_output

    assert await _wait_for_suite_absence(service, nested_suite.id), (
        f"Suite {nested_suite.id} is still present after delete lifecycle retries"
    )


async def test_e2e_hierarchy_delete_parent_suite_with_children(
    project_id: int,
    allure_client: AllureClient,
    cleanup_tracker: CleanupTracker,
) -> None:
    """Delete parent suite with nested suite + assigned case and verify hierarchy cleanup."""
    service = TestHierarchyService(allure_client)
    run_id = uuid4().hex[:8]

    root_name = f"E2E-Delete-Parent-Root-{project_id}-{run_id}"
    nested_name = f"E2E-Delete-Parent-Nested-{project_id}-{run_id}"
    cleanup_tracker.track_custom_field_value_name(root_name)
    cleanup_tracker.track_custom_field_value_name(nested_name)

    root_suite = await service.create_test_suite(name=root_name)
    assert root_suite.id is not None
    cleanup_tracker.track_test_suite(root_suite.id)

    nested_suite = await service.create_test_suite(name=nested_name, parent_suite_id=root_suite.id)
    assert nested_suite.id is not None
    cleanup_tracker.track_test_suite(nested_suite.id)

    tc_output = await create_test_case(name=f"E2E Delete Parent Suite Case {run_id}", project_id=project_id)
    match = re.search(r"ID: (\d+)", tc_output)
    assert match is not None
    test_case_id = int(match.group(1))
    cleanup_tracker.track_test_case(test_case_id)

    assign_output = await assign_test_cases_to_suite(
        suite_id=nested_suite.id,
        test_case_ids=[test_case_id],
        project_id=project_id,
    )
    assert "Assigned 1 test case(s)" in assign_output

    delete_parent_output = await delete_test_suite(
        suite_id=root_suite.id,
        confirm=True,
        project_id=project_id,
    )
    assert delete_parent_output == f"✅ Test suite {root_suite.id} deleted successfully (idempotent)."

    assert await _wait_for_suite_absence(service, root_suite.id), (
        f"Parent suite {root_suite.id} is still present after deletion retries"
    )
    assert await _wait_for_suite_absence(service, nested_suite.id), (
        f"Nested suite {nested_suite.id} is still present after parent deletion retries"
    )



"""Create a hierarchy suite in Allure TestOps.

This tool creates a suite (group node) in a project tree. It can create either:
- a top-level suite under the tree root (`parent_suite_id` omitted), or
- a nested suite under an existing parent suite (`parent_suite_id` provided).

This operation creates new hierarchy structure and is not destructive.
"""

from typing import Annotated

from pydantic import Field

from src.client import AllureClient
from src.services.test_hierarchy_service import TestHierarchyService


async def create_test_suite(
    name: Annotated[str, Field(description="Suite name to create in hierarchy.")],
    project_id: Annotated[int | None, Field(description="Allure TestOps project ID.")] = None,
    tree_id: Annotated[
        int | None, Field(description="Target hierarchy tree ID. If omitted, default project tree is used.")
    ] = None,
    parent_suite_id: Annotated[
        int | None, Field(description="Parent suite/group node ID for nested suite creation.")
    ] = None,
) -> str:
    """Create a new test suite node in the hierarchy tree.

    Args:
        name: Suite name to create.
        project_id: Optional project override. If omitted, use default project from environment.
        tree_id: Optional hierarchy tree ID. If omitted, the default project tree is used.
        parent_suite_id: Optional parent suite ID. Provide to create a nested suite.

    Returns:
        Success message containing created suite ID and name.
    """
    async with AllureClient.from_env(project=project_id) as client:
        service = TestHierarchyService(client)
        suite = await service.create_test_suite(
            name=name,
            tree_id=tree_id,
            parent_suite_id=parent_suite_id,
        )

    return f"✅ Test suite created successfully! ID: {suite.id}, Name: {suite.name}"



import asyncio
import hashlib
import json

import httpx
import pytest
import respx

from src.services.telemetry_service import TelemetryService
from src.utils.config import settings


@pytest.mark.asyncio
@respx.mock
async def test_startup_event_payload_and_user_agent(monkeypatch: pytest.MonkeyPatch) -> None:
    monkeypatch.setattr(settings, "ALLURE_ENDPOINT", "https://example.testops.cloud")
    monkeypatch.setattr(settings, "ALLURE_PROJECT_ID", 123)

    service = TelemetryService(
        enabled=True,
        umami_base_url="https://cloud.umami.is/api/send",
        umami_website_id="website-1",
        umami_hostname="lucius.test",
        hash_salt="telemetry-salt",
        mcp_mode="http",
        server_version="1.2.3",
    )
    route = respx.post("https://cloud.umami.is/api/send").mock(return_value=httpx.Response(204))

    service.emit_startup_event()
    await service.drain()

    assert route.called
    request = route.calls.last.request
    body = json.loads(request.content.decode("utf-8"))
    event_payload = body["payload"]
    payload = event_payload["data"]

    assert request.headers["User-Agent"]
    assert body["type"] == "event"
    assert event_payload["name"] == f"startup.{payload['deployment_method']}"
    assert payload["server_version"] == "1.2.3"
    assert payload["mcp_mode"] == "http"
    assert payload["deployment_method"] in {"docker", "mcpb", "uvx+pypi", "cli", "plain-code-checkout"}
    assert payload["project_id_hash"] != "123"
    assert payload["endpoint_host_hash"] != "example.testops.cloud"


@pytest.mark.asyncio
@respx.mock
async def test_tool_event_error_contains_classification(monkeypatch: pytest.MonkeyPatch) -> None:
    monkeypatch.setattr(settings, "ALLURE_ENDPOINT", "https://example.testops.cloud")
    monkeypatch.setattr(settings, "ALLURE_PROJECT_ID", 321)

    service = TelemetryService(
        enabled=True,
        umami_base_url="https://cloud.umami.is",
        umami_website_id="website-1",
        umami_hostname="lucius.test",
        hash_salt="telemetry-salt",
        mcp_mode="stdio",
    )
    route = respx.post("https://cloud.umami.is/api/send").mock(return_value=httpx.Response(204))

    service.emit_tool_usage_event(
        tool_name="create_test_case",
        outcome="error",
        duration_ms=750.0,
        error=ValueError("invalid"),
    )
    await service.drain()

    assert route.called
    body = json.loads(route.calls.last.request.content.decode("utf-8"))
    event_payload = body["payload"]
    payload = event_payload["data"]
    assert event_payload["name"] == "tool_error.validation.create_test_case"
    assert payload["tool_name"] == "create_test_case"
    assert payload["outcome"] == "error"
    assert payload["duration_bucket"] == "500ms-2s"
    assert payload["error_category"] == "validation"


@pytest.mark.asyncio
@respx.mock
async def test_tool_event_non_validation_error_uses_other_error_event() -> None:
    service = TelemetryService(
        enabled=True,
        umami_base_url="https://cloud.umami.is",
        umami_website_id="website-1",
        umami_hostname="lucius.test",
    )
    route = respx.post("https://cloud.umami.is/api/send").mock(return_value=httpx.Response(204))

    service.emit_tool_usage_event(
        tool_name="search_test_cases",
        outcome="error",
        duration_ms=80.0,
        error=RuntimeError("boom"),
    )
    await service.drain()

    assert route.called
    body = json.loads(route.calls.last.request.content.decode("utf-8"))
    event_payload = body["payload"]
    payload = event_payload["data"]
    assert event_payload["name"] == "tool_error.other.search_test_cases"
    assert payload["error_category"] == "unexpected"


@pytest.mark.asyncio
@respx.mock
async def test_opt_out_disables_http_requests() -> None:
    service = TelemetryService(
        enabled=False,
        umami_base_url="https://cloud.umami.is",
        umami_website_id="website-1",
        umami_hostname="lucius.test",
    )
    route = respx.post("https://cloud.umami.is/api/send").mock(return_value=httpx.Response(204))

    service.emit_startup_event()
    service.emit_tool_usage_event(tool_name="list_test_cases", outcome="success", duration_ms=20.0)
    await service.drain()

    assert not route.called


@pytest.mark.asyncio
@respx.mock
async def test_env_opt_out_disables_http_requests(monkeypatch: pytest.MonkeyPatch) -> None:
    monkeypatch.setattr(settings, "TELEMETRY_ENABLED", False)
    service = TelemetryService(
        umami_base_url="https://cloud.umami.is",
        umami_website_id="website-1",
        umami_hostname="lucius.test",
    )
    route = respx.post("https://cloud.umami.is/api/send").mock(return_value=httpx.Response(204))

    service.emit_startup_event()
    service.emit_tool_usage_event(tool_name="list_test_cases", outcome="success", duration_ms=20.0)
    await service.drain()

    assert service.is_enabled is False
    assert not route.called


def test_env_overrides_umami_website_id_and_hostname(monkeypatch: pytest.MonkeyPatch) -> None:
    monkeypatch.setattr(settings, "TELEMETRY_WEBSITE_ID", "env-website")
    monkeypatch.setattr(settings, "TELEMETRY_HOSTNAME", "env-hostname.test")

    service = TelemetryService(enabled=False)

    assert service._website_id == "env-website"
    assert service._hostname == "env-hostname.test"


def test_explicit_umami_values_override_env(monkeypatch: pytest.MonkeyPatch) -> None:
    monkeypatch.setattr(settings, "TELEMETRY_WEBSITE_ID", "env-website")
    monkeypatch.setattr(settings, "TELEMETRY_HOSTNAME", "env-hostname.test")

    service = TelemetryService(
        enabled=False,
        umami_website_id="explicit-website",
        umami_hostname="explicit-hostname.test",
    )

    assert service._website_id == "explicit-website"
    assert service._hostname == "explicit-hostname.test"


@pytest.mark.asyncio
@respx.mock
async def test_send_failures_are_swallowed(caplog: pytest.LogCaptureFixture) -> None:
    service = TelemetryService(
        enabled=True,
        umami_base_url="https://cloud.umami.is",
        umami_website_id="website-1",
        umami_hostname="lucius.test",
    )
    respx.post("https://cloud.umami.is/api/send").mock(side_effect=httpx.ConnectError("down"))

    service.emit_tool_usage_event(tool_name="list_test_cases", outcome="success", duration_ms=10.0)
    await service.drain()

    assert "Telemetry emission failed" in caplog.text


def test_hash_identifier_is_deterministic() -> None:
    service = TelemetryService(enabled=False, hash_salt="salt")

    hash_a = service._hash_identifier("abc")
    hash_b = service._hash_identifier("abc")
    hash_c = service._hash_identifier("abcd")

    assert hash_a == hash_b
    assert hash_a != hash_c


def test_sensitive_fields_use_salted_hashes(monkeypatch: pytest.MonkeyPatch) -> None:
    monkeypatch.setattr(settings, "ALLURE_ENDPOINT", "https://sensitive.testops.cloud")
    monkeypatch.setattr(settings, "ALLURE_PROJECT_ID", 456)

    service = TelemetryService(enabled=False, hash_salt="telemetry-salt")
    payload = service._runtime_context()

    expected_project_hash = hashlib.sha256(b"telemetry-salt:456").hexdigest()
    expected_host_hash = hashlib.sha256(b"telemetry-salt:sensitive.testops.cloud").hexdigest()

    assert payload["project_id_hash"] == expected_project_hash
    assert payload["endpoint_host_hash"] == expected_host_hash
    assert payload["project_id_hash"] != "456"
    assert payload["endpoint_host_hash"] != "sensitive.testops.cloud"


@pytest.mark.asyncio
async def test_tool_event_is_scheduled_async_non_blocking() -> None:
    service = TelemetryService(
        enabled=True,
        umami_base_url="https://cloud.umami.is",
        umami_website_id="website-1",
        umami_hostname="lucius.test",
    )
    entered_send = asyncio.Event()
    release_send = asyncio.Event()

    async def fake_send_event(*, event_name: str, payload: dict[str, str]) -> None:
        assert event_name == "tool_use.list_test_cases"
        assert payload["tool_name"] == "list_test_cases"
        entered_send.set()
        await release_send.wait()

    service._send_event = fake_send_event  # type: ignore[method-assign]

    service.emit_tool_usage_event(tool_name="list_test_cases", outcome="success", duration_ms=5.0)
    assert len(service._pending_tasks) == 1
    task = next(iter(service._pending_tasks))
    assert not task.done()

    await asyncio.wait_for(entered_send.wait(), timeout=1.0)
    release_send.set()
    await service.drain()
    assert not service._pending_tasks



import logging

from src.client import AllureClient
from src.client.exceptions import AllureAPIError, AllureNotFoundError, AllureValidationError

logger = logging.getLogger(__name__)


class CustomFieldService:
    """Service for project-level custom field cleanup operations."""

    def __init__(self, client: AllureClient) -> None:
        self._client = client
        self._project_id = client.get_project()

    async def cleanup_unused(self, page_size: int = 100) -> int:
        """Remove custom fields that are unused by any test case in this project."""
        if not isinstance(self._project_id, int) or self._project_id <= 0:
            raise AllureValidationError("Project ID must be a positive integer")
        if not isinstance(page_size, int) or page_size <= 0:
            raise AllureValidationError("page_size must be a positive integer")

        project_field_ids = await self._list_project_field_ids(page_size=page_size)

        deleted_count = 0
        for field_id in project_field_ids:
            try:
                if await self._is_field_in_use(field_id):
                    continue

                if await self._remove_field_from_project(field_id):
                    deleted_count += 1
            except AllureAPIError as exc:
                logger.warning(
                    "Failed to process custom field %s in project %s during cleanup: %s",
                    field_id,
                    self._project_id,
                    exc,
                )
                continue

        return deleted_count

    async def _list_project_field_ids(self, page_size: int) -> list[int]:
        field_ids: list[int] = []
        page = 0

        while True:
            fields = await self._client.list_project_custom_fields(
                project_id=self._project_id,
                page=page,
                size=page_size,
            )
            if not fields:
                break

            for field in fields:
                if field.custom_field is None or field.custom_field.id is None:
                    continue
                field_ids.append(field.custom_field.id)

            if len(fields) < page_size:
                break
            page += 1

        return sorted(set(field_ids))

    async def _is_field_in_use(self, field_id: int) -> bool:
        # Evaluate usage in both active and archived test cases to avoid
        # removing fields still referenced in project history.
        for deleted in (False, True):
            usage_stats = await self._client.count_test_cases_in_projects(
                project_ids=[self._project_id],
                custom_field_id=field_id,
                deleted=deleted,
            )

            for stat in usage_stats:
                if stat.id == self._project_id and (stat.test_case_count or 0) > 0:
                    return True
        return False

    async def _remove_field_from_project(self, field_id: int) -> bool:
        try:
            await self._client.remove_custom_field_from_project(
                custom_field_id=field_id,
                project_id=self._project_id,
            )
            return True
        except AllureNotFoundError:
            logger.debug("Custom field %s was already removed from project %s.", field_id, self._project_id)
            return False



import pytest
from starlette.applications import Starlette
from starlette.routing import Mount

from src.utils.config import settings

settings.MCP_MODE = "http"

# Fixtures are imported via pytest_plugins to avoid re-import warnings
pytest_plugins = [
    "tests.support.fixtures.base",
    "tests.support.fixtures.logger_fixture",
    "tests.support.fixtures.client_fixture",
    "tests.support.fixtures.allure_client_fixture",
]


@pytest.fixture
def app() -> Starlette:
    """
    Fixture that returns the Starlette application with a FRESH FastMCP session manager.
    FastMCP's StreamableHTTPSessionManager is single-use, so we must recreate
    the mcp_asgi app and update the global app references for each test.
    """
    import src.main

    # 1. Generate a new ASGI app from the global mcp instance
    # This creates a new StreamableHTTPSessionManager
    new_asgi = src.main.mcp.http_app()

    # 2. Update the internal variable so get_mcp_asgi() and subsequent calls use the new one
    src.main._mcp_asgi = new_asgi

    # 3. Update the Starlette app's routes to point to the new ASGI app
    # We find the Mount for "/" and replace it
    # Note: Accessing/modifying routes list directly.
    # The routes list contains Route or Mount objects.
    for i, route in enumerate(src.main.app.routes):
        if isinstance(route, Mount) and route.path == "/":
            src.main.app.routes[i] = Mount("/", app=new_asgi)
            break

    return src.main.app



from unittest.mock import AsyncMock, MagicMock

import pytest

from src.client import (
    AllureClient,
    PageSharedStepDto,
    SharedStepAttachmentRowDto,
    SharedStepDto,
)
from src.client.exceptions import AllureValidationError
from src.services.shared_step_service import SharedStepService


@pytest.fixture
def mock_client():
    client = MagicMock(spec=AllureClient)
    client.create_shared_step = AsyncMock()
    client.create_shared_step_scenario_step = AsyncMock()
    client.patch_shared_step_scenario_step = AsyncMock()
    client.list_shared_steps = AsyncMock()
    client.upload_shared_step_attachment = AsyncMock()
    client.get_project.return_value = 1
    return client


@pytest.fixture
def service(mock_client):
    return SharedStepService(client=mock_client)


@pytest.mark.asyncio
async def test_create_shared_step_success(service, mock_client):
    """Test creating a shared step with steps and attachments."""
    mock_client.create_shared_step.return_value = SharedStepDto(id=100, name="Shared Step", project_id=1)
    # Use generic objects for step response to avoid oneOf validation issues during test
    action_mock = MagicMock()
    action_mock.created_step_id = 201

    expected_mock = MagicMock()
    expected_mock.created_step_id = 202

    att_mock = MagicMock()
    att_mock.created_step_id = 203

    mock_client.create_shared_step_scenario_step.side_effect = [
        action_mock,  # Action
        expected_mock,  # Expected
        att_mock,  # Attachment step
    ]
    mock_client.upload_shared_step_attachment.return_value = [SharedStepAttachmentRowDto(id=500, name="file.txt")]

    steps = [
        {
            "action": "Do something",
            "expected": "Something happens",
            "attachments": [{"name": "file.txt", "content": "Zm9vYmFy"}],
        }
    ]

    result = await service.create_shared_step(name="Shared Step", steps=steps)

    assert result.id == 100
    mock_client.create_shared_step.assert_called_once_with(1, "Shared Step")

    # Check step creation calls
    assert mock_client.create_shared_step_scenario_step.call_count == 3
    mock_client.patch_shared_step_scenario_step.assert_called_once()

    # 1. Action
    action_call = mock_client.create_shared_step_scenario_step.call_args_list[0]
    assert action_call[0][0].body == "Do something"
    assert action_call[0][0].shared_step_id == 100

    # Patch expected result on action step
    patch_call = mock_client.patch_shared_step_scenario_step.call_args
    assert patch_call.kwargs["step_id"] == 201
    assert patch_call.kwargs["patch"].body == "Do something"
    assert patch_call.kwargs["patch"].expected_result == "Something happens"

    # 2. Expected
    expected_call = mock_client.create_shared_step_scenario_step.call_args_list[1]
    assert expected_call[0][0].body == "Something happens"
    assert expected_call[0][0].parent_id == 201

    # 3. Attachment
    att_call = mock_client.create_shared_step_scenario_step.call_args_list[2]
    assert att_call[0][0].attachment_id == 500
    assert att_call[0][0].parent_id == 201


@pytest.mark.asyncio
async def test_list_shared_steps_success(service, mock_client):
    """Test listing shared steps."""
    mock_page = PageSharedStepDto(
        content=[
            SharedStepDto(id=1, name="Step 1", steps_count=5),
            SharedStepDto(id=2, name="Step 2", steps_count=3),
        ],
        total_elements=2,
    )
    mock_client.list_shared_steps.return_value = mock_page

    result = await service.list_shared_steps(search="Step")

    assert len(result) == 2
    assert result[0].name == "Step 1"
    mock_client.list_shared_steps.assert_called_once_with(project_id=1, page=0, size=100, search="Step", archived=None)


@pytest.mark.asyncio
async def test_create_shared_step_fail_validation(service):
    """Test validation errors."""
    service._project_id = -1
    with pytest.raises(AllureValidationError):
        await service.create_shared_step(name="Valid")

    service._project_id = 1
    with pytest.raises(AllureValidationError):
        await service.create_shared_step(name="")


# ==========================================
# Update Shared Step Tests (Story 2.2)
# ==========================================


@pytest.mark.asyncio
async def test_get_shared_step_success(service, mock_client):
    """Test getting a shared step."""
    expected_step = SharedStepDto(id=100, name="Test Step", project_id=1, steps_count=3)
    mock_client.get_shared_step = AsyncMock(return_value=expected_step)

    result = await service.get_shared_step(step_id=100)

    assert result.id == 100
    assert result.name == "Test Step"
    mock_client.get_shared_step.assert_called_once_with(100)


@pytest.mark.priority("P0")
@pytest.mark.asyncio
async def test_update_shared_step_success(service, mock_client):
    """
    Test updating a shared step with name changes.
    ID: 2.2-UNIT-001
    """
    current_step = SharedStepDto(id=100, name="Old Name", project_id=1)
    updated_step = SharedStepDto(id=100, name="New Name", project_id=1)

    mock_client.get_shared_step = AsyncMock(return_value=current_step)
    mock_client.update_shared_step = AsyncMock(return_value=updated_step)

    result, changed = await service.update_shared_step(step_id=100, name="New Name")

    assert result.name == "New Name"
    assert changed is True
    mock_client.get_shared_step.assert_called_once_with(100)
    mock_client.update_shared_step.assert_called_once()


@pytest.mark.priority("P2")
@pytest.mark.asyncio
async def test_update_shared_step_idempotent(service, mock_client):
    """
    Test idempotency - no update if data matches current state.
    ID: 2.2-UNIT-002
    """
    current_step = SharedStepDto(id=100, name="Same Name", project_id=1)

    mock_client.get_shared_step = AsyncMock(return_value=current_step)
    mock_client.update_shared_step = AsyncMock()

    result, changed = await service.update_shared_step(step_id=100, name="Same Name")

    assert result.name == "Same Name"
    assert changed is False
    mock_client.get_shared_step.assert_called_once_with(100)
    # Should NOT call update_shared_step
    mock_client.update_shared_step.assert_not_called()


@pytest.mark.priority("P2")
@pytest.mark.asyncio
async def test_update_shared_step_partial_update(service, mock_client):
    """
    Test partial update with only some fields provided.
    ID: 2.2-UNIT-003
    """
    current_step = SharedStepDto(id=100, name="Current Name", project_id=1, description="Old Desc")
    updated_step = SharedStepDto(id=100, name="Current Name", project_id=1, description="New Desc")

    mock_client.get_shared_step = AsyncMock(return_value=current_step)
    mock_client.update_shared_step = AsyncMock(return_value=updated_step)

    # Only update description, leave name unchanged
    result, changed = await service.update_shared_step(step_id=100, description="New Desc")

    assert result is not None
    assert changed is True
    mock_client.update_shared_step.assert_called_once()


# ==========================================
# Delete Shared Step Tests (Story 2.2)
# ==========================================


@pytest.mark.priority("P0")
@pytest.mark.asyncio
async def test_delete_shared_step_success(service, mock_client):
    """
    Test deleting a shared step successfully.
    ID: 2.2-UNIT-004
    """
    mock_client.get_shared_step = AsyncMock(
        return_value=SharedStepDto(id=100, name="Test Step", project_id=1, steps_count=3)
    )
    mock_client.delete_shared_step = AsyncMock()

    result = await service.delete_shared_step(step_id=100)

    assert result is True
    mock_client.get_shared_step.assert_called_once_with(100)
    mock_client.delete_shared_step.assert_called_once_with(100)


@pytest.mark.priority("P2")
@pytest.mark.asyncio
async def test_delete_shared_step_idempotent(service, mock_client):
    """
    Test delete is idempotent (soft delete pattern from Story 1.5).
    ID: 2.2-UNIT-005
    """
    from src.client.exceptions import AllureNotFoundError

    # Mock get_shared_step to fail (already deleted)
    mock_client.get_shared_step = AsyncMock(side_effect=AllureNotFoundError("Not found", 404, "{}"))
    mock_client.delete_shared_step = AsyncMock()

    # Should not raise, just return False
    result = await service.delete_shared_step(step_id=100)

    assert result is False
    mock_client.get_shared_step.assert_called_once_with(100)
    mock_client.delete_shared_step.assert_not_called()



from typing import Annotated

from pydantic import Field

from src.client import AllureClient
from src.services.custom_field_value_service import CustomFieldValueService


async def list_custom_field_values(
    custom_field_id: Annotated[
        int | None, Field(description="Project-scoped custom field ID to list values for.")
    ] = None,
    custom_field_name: Annotated[
        str | None,
        Field(description="Custom field name to resolve when custom_field_id is not provided."),
    ] = None,
    query: Annotated[str | None, Field(description="Optional search query to filter values.")] = None,
    var_global: Annotated[bool | None, Field(description="Optional filter for global values.")] = None,
    test_case_search: Annotated[
        str | None,
        Field(description="Optional test case search filter to match values in test cases."),
    ] = None,
    page: Annotated[int | None, Field(description="Zero-based page index.")] = None,
    size: Annotated[int | None, Field(description="Number of items per page.")] = None,
    sort: Annotated[
        list[str] | None,
        Field(description="Optional sort criteria, e.g. ['name,asc', 'id,desc']"),
    ] = None,
    project_id: Annotated[int | None, Field(description="Optional override for the default Project ID.")] = None,
) -> str:
    """List available values for a custom field.

    Args:
        custom_field_id: Project-scoped custom field ID to list values for.
        custom_field_name: Custom field name to resolve when custom_field_id is not provided.
        query: Optional search query to filter values.
        var_global: Optional filter for global values.
        test_case_search: Optional test case search filter to match values in test cases.
        page: Zero-based page index.
        size: Number of items per page.
        sort: Optional sort criteria, e.g. ["name,asc", "id,desc"].
        project_id: Optional override for the default Project ID.
    """
    async with AllureClient.from_env(project=project_id) as client:
        service = CustomFieldValueService(client=client)
        result = await service.list_custom_field_values(
            project_id=project_id,
            custom_field_id=custom_field_id,
            custom_field_name=custom_field_name,
            query=query,
            var_global=var_global,
            test_case_search=test_case_search,
            page=page,
            size=size,
            sort=sort,
        )

        values = result.content or []
        if not values:
            return "No custom field values found."

        total = result.total_elements if result.total_elements is not None else len(values)
        header = f"Found {total} custom field values:"
        if result.number is not None and result.total_pages is not None:
            header = f"Found {total} custom field values (page {result.number + 1}/{result.total_pages}):"

        lines = [header]
        for value in values:
            count = value.test_cases_count if value.test_cases_count is not None else 0
            lines.append(f"- ID: {value.id}, Name: {value.name}, Test cases: {count}")

        return "\n".join(lines)



"""Unit tests for IntegrationService."""

from unittest.mock import AsyncMock

import pytest

from src.client.exceptions import AllureAPIError, AllureNotFoundError, AllureValidationError
from src.client.generated.models.integration_dto import IntegrationDto
from src.client.generated.models.integration_info_dto import IntegrationInfoDto
from src.services.integration_service import IntegrationService


@pytest.fixture
def mock_client():
    return AsyncMock()


@pytest.fixture
def service(mock_client):
    return IntegrationService(mock_client)


@pytest.fixture
def integrations():
    return [
        IntegrationDto(id=1, name="Jira", info=IntegrationInfoDto(type="jira")),
        IntegrationDto(id=2, name="GitHub", info=IntegrationInfoDto(type="github")),
    ]


@pytest.mark.asyncio
async def test_list_integrations(service, mock_client, integrations):
    mock_client.get_integrations.return_value = integrations
    result = await service.list_integrations()
    assert result == integrations
    mock_client.get_integrations.assert_called_once()


@pytest.mark.asyncio
async def test_list_integrations_project_scoped(service, mock_client, integrations):
    mock_client.get_project_available_integrations.return_value = [integrations[1]]
    result = await service.list_integrations(project_id=42)
    assert result == [integrations[1]]
    mock_client.get_project_available_integrations.assert_called_once_with(42)


@pytest.mark.asyncio
async def test_get_integration_by_id_success(service, mock_client, integrations):
    mock_client.get_integrations.return_value = integrations
    result = await service.get_integration_by_id(1)
    assert result.id == 1
    assert result.name == "Jira"


@pytest.mark.asyncio
async def test_get_integration_by_id_not_found(service, mock_client, integrations):
    mock_client.get_integrations.return_value = integrations
    with pytest.raises(AllureNotFoundError, match="Integration with ID 999 not found"):
        await service.get_integration_by_id(999)


@pytest.mark.asyncio
async def test_get_integration_by_id_invalid(service):
    with pytest.raises(AllureValidationError, match="Integration ID must be a positive integer"):
        await service.get_integration_by_id(0)


@pytest.mark.asyncio
async def test_get_integration_by_name_success(service, mock_client, integrations):
    mock_client.get_integrations.return_value = integrations
    result = await service.get_integration_by_name("GitHub")
    assert result.id == 2
    assert result.info.type == "github"


@pytest.mark.asyncio
async def test_get_integration_by_name_not_found(service, mock_client, integrations):
    mock_client.get_integrations.return_value = integrations
    with pytest.raises(AllureNotFoundError, match="Integration 'GitLab' not found"):
        await service.get_integration_by_name("GitLab")


@pytest.mark.asyncio
async def test_resolve_integration_mutual_exclusivity(service):
    with pytest.raises(AllureValidationError, match="Cannot specify both 'integration_id' and 'integration_name'"):
        await service.resolve_integration(integration_id=1, integration_name="Jira")


@pytest.mark.asyncio
async def test_resolve_integration_by_id(service, mock_client, integrations):
    mock_client.get_integrations.return_value = integrations
    result = await service.resolve_integration(integration_id=2)
    assert result.id == 2


@pytest.mark.asyncio
async def test_resolve_integration_by_name(service, mock_client, integrations):
    mock_client.get_integrations.return_value = integrations
    result = await service.resolve_integration(integration_name="Jira")
    assert result.id == 1


@pytest.mark.asyncio
async def test_resolve_integration_none(service):
    result = await service.resolve_integration()
    assert result is None


@pytest.mark.asyncio
async def test_resolve_integration_for_issues_single(service, mock_client):
    integrations = [IntegrationDto(id=1, name="Jira")]
    mock_client.get_integrations.return_value = integrations
    result = await service.resolve_integration_for_issues()
    assert result == 1


@pytest.mark.asyncio
async def test_resolve_integration_for_issues_project_filtered_single(service, mock_client):
    mock_client.get_project_available_integrations.return_value = [IntegrationDto(id=9, name="Project Jira")]
    result = await service.resolve_integration_for_issues(project_id=7)
    assert result == 9
    mock_client.get_project_available_integrations.assert_called_once_with(7)


@pytest.mark.asyncio
async def test_resolve_integration_for_issues_multiple_error(service, mock_client, integrations):
    mock_client.get_integrations.return_value = integrations
    with pytest.raises(AllureValidationError, match="Multiple integrations found"):
        await service.resolve_integration_for_issues()


@pytest.mark.asyncio
async def test_resolve_integration_for_issues_explicit_override(service, mock_client, integrations):
    mock_client.get_integrations.return_value = integrations
    result = await service.resolve_integration_for_issues(integration_name="GitHub")
    assert result == 2


@pytest.mark.asyncio
async def test_resolve_integration_for_issues_no_integrations(service, mock_client):
    mock_client.get_integrations.return_value = []
    with pytest.raises(AllureAPIError, match="No integrations configured in Allure TestOps"):
        await service.resolve_integration_for_issues()



from unittest.mock import AsyncMock, Mock

import pytest

from src.client import AllureClient
from src.client.exceptions import AllureValidationError
from src.client.generated.models import (
    TestCaseScenarioV2Dto,
)
from src.services.attachment_service import AttachmentService
from src.services.test_case_service import TestCaseService


@pytest.fixture
def mock_client() -> AsyncMock:
    client = AsyncMock(spec=AllureClient)
    client.api_client = Mock()
    client.get_project.return_value = 1
    return client


@pytest.fixture
def mock_attachment_service() -> AsyncMock:
    return AsyncMock(spec=AttachmentService)


@pytest.fixture
def service(mock_client: AsyncMock, mock_attachment_service: AsyncMock) -> TestCaseService:
    return TestCaseService(
        client=mock_client,
        attachment_service=mock_attachment_service,
    )


class TestAddSharedStepToCase:
    """Tests for add_shared_step_to_case."""

    @pytest.mark.asyncio
    async def test_add_shared_step_append_default(
        self, service: TestCaseService, mock_client: AsyncMock, step_dto_factory
    ) -> None:
        """Test appending shared step (position=None)."""
        test_case_id = 100
        shared_step_id = 555

        # Setup existing steps
        existing_step = step_dto_factory(step_type="body", id=10, body="Step 1")
        mock_client.get_test_case_scenario.return_value = TestCaseScenarioV2Dto(steps=[existing_step])

        await service.add_shared_step_to_case(test_case_id, shared_step_id, position=None)

        # Verify call: after_id should be last step ID (10)
        mock_client.create_scenario_step.assert_called_once()
        kwargs = mock_client.create_scenario_step.call_args.kwargs
        assert kwargs["test_case_id"] == test_case_id
        assert kwargs["step"].shared_step_id == shared_step_id
        assert kwargs["after_id"] == 10

    @pytest.mark.asyncio
    async def test_add_shared_step_append_empty(self, service: TestCaseService, mock_client: AsyncMock) -> None:
        """Test appending shared step to empty case."""
        test_case_id = 100
        shared_step_id = 555

        mock_client.get_test_case_scenario.return_value = TestCaseScenarioV2Dto(steps=[])

        await service.add_shared_step_to_case(test_case_id, shared_step_id, position=None)

        # Verify call: after_id should be None (no steps)
        mock_client.create_scenario_step.assert_called_once()
        kwargs = mock_client.create_scenario_step.call_args.kwargs
        assert kwargs["after_id"] is None

    @pytest.mark.asyncio
    async def test_add_shared_step_at_start(
        self, service: TestCaseService, mock_client: AsyncMock, step_dto_factory
    ) -> None:
        """Test inserting shared step at start (position=0)."""
        test_case_id = 100
        shared_step_id = 555

        existing_step = step_dto_factory(step_type="body", id=10, body="Step 1")
        mock_client.get_test_case_scenario.return_value = TestCaseScenarioV2Dto(steps=[existing_step])

        await service.add_shared_step_to_case(test_case_id, shared_step_id, position=0)

        # Verify call: after_id should be None (insert before first step/at start)
        mock_client.create_scenario_step.assert_called_once()
        kwargs = mock_client.create_scenario_step.call_args.kwargs
        assert kwargs["after_id"] is None

    @pytest.mark.asyncio
    async def test_add_shared_step_at_index(
        self, service: TestCaseService, mock_client: AsyncMock, step_dto_factory
    ) -> None:
        """Test inserting shared step at specific index."""
        test_case_id = 100
        shared_step_id = 555

        # Steps: [ID 10, ID 20]
        step1 = step_dto_factory(step_type="body", id=10, body="S1")
        step2 = step_dto_factory(step_type="body", id=20, body="S2")
        mock_client.get_test_case_scenario.return_value = TestCaseScenarioV2Dto(steps=[step1, step2])

        # Insert at index 1 (between 10 and 20)
        # Position 1 means "become the step at index 1". Original index 1 (ID 20) moves to index 2.
        # So we insert AFTER index 0 (ID 10).
        await service.add_shared_step_to_case(test_case_id, shared_step_id, position=1)

        mock_client.create_scenario_step.assert_called_once()
        kwargs = mock_client.create_scenario_step.call_args.kwargs
        assert kwargs["after_id"] == 10

    @pytest.mark.asyncio
    async def test_add_shared_step_position_out_of_bounds(
        self, service: TestCaseService, mock_client: AsyncMock
    ) -> None:
        """Test position out of bounds validation."""
        test_case_id = 100
        mock_client.get_test_case_scenario.return_value = TestCaseScenarioV2Dto(steps=[])

        with pytest.raises(AllureValidationError, match="out of bounds"):
            await service.add_shared_step_to_case(test_case_id, 555, position=1)  # Length 0, index 1 invalid


class TestRemoveSharedStepFromCase:
    """Tests for remove_shared_step_from_case."""

    @pytest.mark.asyncio
    async def test_remove_shared_step_success(
        self, service: TestCaseService, mock_client: AsyncMock, step_dto_factory
    ) -> None:
        """Test removing existing shared step references."""
        test_case_id = 100
        shared_step_id = 555

        # Steps: [Inline(10), Shared(20, ref=555)]
        step1 = step_dto_factory(step_type="body", id=10)
        step2 = step_dto_factory(step_type="shared", id=20, shared_step_id=555)

        mock_client.get_test_case_scenario.return_value = TestCaseScenarioV2Dto(steps=[step1, step2])

        await service.remove_shared_step_from_case(test_case_id, shared_step_id)

        mock_client.delete_scenario_step.assert_called_once_with(20)

    @pytest.mark.asyncio
    async def test_remove_shared_step_multiple_references(
        self, service: TestCaseService, mock_client: AsyncMock, step_dto_factory
    ) -> None:
        """Test removing multiple references to same shared step."""
        test_case_id = 100
        shared_step_id = 555

        # Steps: [Shared(20, ref=555), Shared(30, ref=555)]
        step1 = step_dto_factory(step_type="shared", id=20, shared_step_id=555)
        step2 = step_dto_factory(step_type="shared", id=30, shared_step_id=555)

        mock_client.get_test_case_scenario.return_value = TestCaseScenarioV2Dto(steps=[step1, step2])

        await service.remove_shared_step_from_case(test_case_id, shared_step_id)

        assert mock_client.delete_scenario_step.call_count == 2
        mock_client.delete_scenario_step.assert_any_call(20)
        mock_client.delete_scenario_step.assert_any_call(30)

    @pytest.mark.asyncio
    async def test_remove_shared_step_not_found(
        self, service: TestCaseService, mock_client: AsyncMock, step_dto_factory
    ) -> None:
        """Test validation error when shared step not found."""
        test_case_id = 100
        # Provide steps so it doesn't return early
        step1 = step_dto_factory(step_type="body", id=10)
        mock_client.get_test_case_scenario.return_value = TestCaseScenarioV2Dto(steps=[step1])

        with pytest.raises(AllureValidationError, match="not found in Test Case"):
            await service.remove_shared_step_from_case(test_case_id, 999)



"""Authentication context helpers."""

from __future__ import annotations

from dataclasses import dataclass

from pydantic import SecretStr

from src.utils.config import settings
from src.utils.error import AuthenticationError


@dataclass(frozen=True)
class AuthContext:
    """Authentication context for API operations.

    Supports both environment-based and runtime authentication.
    Runtime values take precedence over environment values.

    Attributes:
        api_token: The Allure TestOps API token (masked in logs).
        project_id: Optional default project ID for operations.
    """

    api_token: SecretStr
    project_id: int | None = None

    @classmethod
    def from_environment(cls) -> AuthContext:
        """Create context from environment variables.

        Environment Variables:
            ALLURE_API_TOKEN: Required API authentication token.
            ALLURE_PROJECT_ID: Optional default project ID.

        Raises:
            AuthenticationError: If ALLURE_API_TOKEN is not set.
        """
        if not settings.ALLURE_API_TOKEN:
            raise AuthenticationError(
                "No API token configured. Set ALLURE_API_TOKEN environment variable or provide api_token argument."
            )

        return cls(api_token=settings.ALLURE_API_TOKEN, project_id=settings.ALLURE_PROJECT_ID)

    def with_overrides(
        self,
        api_token: str | None = None,
        project_id: int | None = None,
    ) -> AuthContext:
        """Create new context with runtime overrides.

        Returns a new AuthContext with specified values overridden.
        Original context is not modified.

        Args:
            api_token: Override the API token.
            project_id: Override the project ID.

        Returns:
            New AuthContext with overrides applied.
        """
        return AuthContext(
            api_token=SecretStr(api_token) if api_token else self.api_token,
            project_id=project_id if project_id is not None else self.project_id,
        )


def get_auth_context(
    api_token: str | None = None,
    project_id: int | None = None,
) -> AuthContext:
    """Get authentication context with optional runtime overrides.

    Resolution order:
    1. Runtime arguments (highest priority)
    2. Environment variables (fallback)

    This function is stateless - each call creates a fresh context.
    Overrides from one call do NOT affect subsequent calls.

    Args:
        api_token: Optional runtime API token override.
        project_id: Optional runtime project ID override.

    Returns:
        AuthContext ready for API operations.

    Raises:
        AuthenticationError: If no token available from any source.
    """
    try:
        base_context = AuthContext.from_environment()
    except AuthenticationError:
        if not api_token:
            raise AuthenticationError(
                "Authentication required. Either:\n"
                "1. Set ALLURE_API_TOKEN environment variable, or\n"
                "2. Provide api_token argument to this tool"
            ) from None
        base_context = AuthContext(api_token=SecretStr(api_token), project_id=project_id)

    if api_token or project_id is not None:
        return base_context.with_overrides(api_token=api_token, project_id=project_id)

    return base_context



import logging

from src.client import AllureClient
from src.client.exceptions import AllureNotFoundError, AllureValidationError
from src.client.generated.api.test_plan_controller_api import TestPlanControllerApi
from src.client.generated.models.test_plan_create_dto import TestPlanCreateDto
from src.client.generated.models.test_plan_dto import TestPlanDto
from src.client.generated.models.test_plan_patch_dto import TestPlanPatchDto
from src.client.generated.models.tree_selection_dto import TreeSelectionDto

logger = logging.getLogger(__name__)


class PlanService:
    """Service for managing Test Plans in Allure TestOps."""

    def __init__(self, client: AllureClient) -> None:
        self._client = client
        self._project_id = client.get_project()

    @property
    def _api(self) -> TestPlanControllerApi:
        return TestPlanControllerApi(self._client.api_client)

    async def create_plan(
        self,
        name: str,
        test_case_ids: list[int] | None = None,
        aql_filter: str | None = None,
    ) -> TestPlanDto:
        """Create a new test plan.

        Args:
            name: Plan name.
            test_case_ids: List of specific test case IDs to include.
            aql_filter: AQL query to select test cases.

        Returns:
            Created Test Plan DTO.
        """
        if not name:
            raise AllureValidationError("Test Plan name is required")

        if not test_case_ids and not aql_filter:
            # Allow empty plans, but log warning if neither provided?
            # Story doesn't strictly forbid empty plans, useful for setting up structure.
            pass

        create_dto = TestPlanCreateDto(
            name=name,
            project_id=self._project_id,
        )

        plan = await self._client._call_api(self._api.create7(test_plan_create_dto=create_dto))

        # Apply selection/filter if needed
        if test_case_ids or aql_filter:
            # Ensure plan has an ID
            if plan.id is None:
                raise AllureValidationError("Created plan returned no ID")

            await self.update_plan_content(plan_id=plan.id, test_case_ids_add=test_case_ids, aql_filter=aql_filter)
            # Re-fetch
            plan = await self.get_plan(plan.id)

        return plan

    async def get_plan(self, plan_id: int) -> TestPlanDto:
        """Get a test plan by ID."""
        return await self._client._call_api(self._api.find_one6(id=plan_id))

    async def update_plan(
        self,
        plan_id: int,
        name: str | None = None,
    ) -> TestPlanDto:
        """Update a test plan's metadata."""
        await self.get_plan(plan_id)

        # Build Patch DTO
        # Note: TestPlanPatchDto might not have all fields (e.g. description/tags might separate?)
        patch = TestPlanPatchDto(
            name=name,
        )

        # If patch has keys (ignoring None), call update
        if patch.to_dict():
            await self._client._call_api(self._api.patch7(id=plan_id, test_plan_patch_dto=patch))

        return await self.get_plan(plan_id)

    async def update_plan_content(
        self,
        plan_id: int,
        test_case_ids_add: list[int] | None = None,
        test_case_ids_remove: list[int] | None = None,
        aql_filter: str | None = None,
    ) -> TestPlanDto:
        """Update plan content (selection/filter)."""
        current = await self.get_plan(plan_id)

        current_selection = current.tree_selection or TreeSelectionDto()
        current_leafs_include = set(current_selection.leafs_include or [])
        current_leafs_exclude = set(current_selection.leafs_exclude or [])

        if test_case_ids_add:
            current_leafs_include.update(test_case_ids_add)
            # If added, ensure not in exclude
            current_leafs_exclude.difference_update(test_case_ids_add)

        if test_case_ids_remove:
            # If in include, remove
            current_leafs_include.difference_update(test_case_ids_remove)
            # If the plan has a base_rql, removing a case means adding to exclude.
            if current.base_rql or aql_filter:
                current_leafs_exclude.update(test_case_ids_remove)

        new_selection = TreeSelectionDto(
            leafs_include=list(current_leafs_include),
            leafs_exclude=list(current_leafs_exclude),
            groups_include=current_selection.groups_include,
            groups_exclude=current_selection.groups_exclude,
            inverted=current_selection.inverted,
            path=current_selection.path,
        )

        patch = TestPlanPatchDto(
            tree_selection=new_selection, base_rql=aql_filter if aql_filter is not None else current.base_rql
        )

        await self._client._call_api(self._api.patch7(id=plan_id, test_plan_patch_dto=patch))

        return await self.get_plan(plan_id)

    async def add_cases_to_plan(self, plan_id: int, test_case_ids: list[int]) -> None:
        """Add test cases to a plan."""
        await self.update_plan_content(plan_id, test_case_ids_add=test_case_ids)

    async def remove_cases_from_plan(self, plan_id: int, test_case_ids: list[int]) -> None:
        """Remove test cases from a plan."""
        await self.update_plan_content(plan_id, test_case_ids_remove=test_case_ids)

    async def list_plans(self, page: int = 0, size: int = 100) -> list[TestPlanDto]:
        """List test plans."""
        response = await self._client._call_api(
            self._api.find_all_by_project(project_id=self._project_id, page=page, size=size, sort=["id,desc"])
        )
        return response.content or []

    async def delete_plan(self, plan_id: int) -> None:
        """Delete a test plan."""
        try:
            await self._client._call_api(self._api.delete7(id=plan_id))
        except AllureNotFoundError:
            # Idempotent behavior: if plan is already gone, consider it success
            logger.info("Test plan %s not found during deletion (treated as success)", plan_id)



import json

# Configuration
INPUT_FILE = "openapi/allure-testops-service/report-service.json"
OUTPUT_FILE = "openapi/allure-testops-service/filtered-report-service.json"

# Tags to keep (MVP + Essential Supporting Controllers)
KEEP_TAGS = {
    # Core Entities
    "test-case-controller",
    "shared-step-controller",
    "project-controller",
    "launch-controller",
    # Test Case Details
    "test-case-attachment-controller",
    "test-case-scenario-controller",
    "test-case-tag-controller",
    "test-case-overview-controller",
    "test-case-custom-field-controller",
    # Shared Step Details
    "shared-step-attachment-controller",
    "shared-step-scenario-controller",
    # Search & Bulk
    "test-case-search-controller",
    "launch-search-controller",
    "test-case-bulk-controller",
    # Project & Custom Field Management
    "custom-field-controller",
    "custom-field-project-controller",
    "custom-field-project-controller-v-2",
    "custom-field-schema-controller",
    "custom-field-value-controller",
    "custom-field-value-project-controller",
    "status-controller",
    # Test Layer Management
    "test-layer-controller",
    "test-layer-schema-controller",
    # Test Hierarchy Management
    "test-case-tree-controller-v-2",
    "tree-controller-v-2",
    "test-case-tree-bulk-controller-v-2",
    "integration-controller",
    # Test Plan Management
    "test-plan-controller",
    "test-case-test-plan-bulk-controller",
    # Defect Management
    "defect-controller",
    "defect-matcher-controller",
}


def filter_spec() -> None:
    print(f"Reading spec from {INPUT_FILE}...")
    with open(INPUT_FILE) as f:
        spec = json.load(f)

    original_paths_count = len(spec.get("paths", {}))
    print(f"Original paths: {original_paths_count}")

    # Filter paths
    filtered_paths = {}
    for path, methods in spec.get("paths", {}).items():
        new_methods = {}
        for method, operation in methods.items():
            if method == "parameters":  # Keep path-level parameters if any
                new_methods[method] = operation
                continue

            tags = operation.get("tags", [])
            # Check if any of the operation's tags are in our keep list
            if any(tag in KEEP_TAGS for tag in tags):
                new_methods[method] = operation

        if new_methods:
            filtered_paths[path] = new_methods

    spec["paths"] = filtered_paths

    # We are NOT filtering components/schemas aggressively because it's hard to trace
    # all dependencies (refs) without a full graph traversal.
    # openapi-python-client might be smart enough to only generate models that are used,
    # or we can accept the extra models as "future proofing" without the bloat of 100+ extra controllers.

    print(f"Filtered paths: {len(filtered_paths)}")
    print(f"Writing filtered spec to {OUTPUT_FILE}...")

    with open(OUTPUT_FILE, "w") as f:
        json.dump(spec, f, indent=2)


if __name__ == "__main__":
    filter_spec()



"""Telemetry collection service with privacy-preserving payload shaping."""

from __future__ import annotations

import asyncio
import datetime as dt
import hashlib
import os
import platform
import sys
import uuid
from pathlib import Path
from typing import Literal
from urllib.parse import urlparse

import httpx
import umami  # type: ignore[import-untyped]
from pydantic import ValidationError

from src.utils.config import settings, telemetry_config
from src.utils.logger import get_logger
from src.version import __version__

logger = get_logger(__name__)

type TelemetryOutcome = Literal["success", "error"]
type TelemetryErrorCategory = Literal["validation", "auth", "api", "unexpected"]
type DeploymentMethod = Literal["docker", "mcpb", "uvx+pypi", "cli", "plain-code-checkout"]
type MpcMode = Literal["stdio", "http"]


class TelemetryService:
    """Collect and send best-effort telemetry events."""

    def __init__(
        self,
        *,
        enabled: bool | None = None,
        umami_base_url: str | None = None,
        umami_website_id: str | None = None,
        umami_hostname: str | None = None,
        hash_salt: str | None = None,
        mcp_mode: MpcMode | None = None,
        server_version: str = __version__,
    ) -> None:
        self._enabled = self._resolve_enabled(enabled)
        self._umami_base_url = self._resolve_umami_base_url(
            telemetry_config.umami_base_url if umami_base_url is None else umami_base_url
        )
        self._website_id = self._resolve_umami_website_id(umami_website_id)
        self._hostname = self._resolve_umami_hostname(umami_hostname)
        self._hash_salt = telemetry_config.hash_salt if hash_salt is None else hash_salt
        self._mcp_mode = settings.MCP_MODE if mcp_mode is None else mcp_mode
        self._server_version = server_version
        self._session_id = uuid.uuid4().hex
        self._started_at = dt.datetime.now(dt.UTC).isoformat()
        self._pending_tasks: set[asyncio.Task[None]] = set()
        self._missing_website_warned = False
        self._umami_ready = self._configure_umami_client()

    @property
    def is_enabled(self) -> bool:
        return self._enabled

    def log_status(self) -> None:
        status = "enabled" if self._enabled else "disabled"
        logger.info("Telemetry status: %s", status)

    def emit_startup_event(self) -> None:
        if not self._enabled:
            return
        deployment_method = self._detect_deployment_method()
        payload = self._runtime_context(deployment_method=deployment_method)
        session_id_hash = self._hash_identifier(self._session_id) or "unknown"
        payload.update(
            {
                "session_id_hash": session_id_hash,
                "process_started_at": self._started_at,
                "installation_id_hash": self._installation_id_hash(),
            }
        )
        self._schedule_event(f"startup.{deployment_method}", payload)

    def emit_tool_usage_event(
        self,
        *,
        tool_name: str,
        outcome: TelemetryOutcome,
        duration_ms: float,
        error: Exception | None = None,
    ) -> None:
        if not self._enabled:
            return
        payload = self._runtime_context()
        event_name = f"tool_use.{tool_name}"
        payload.update(
            {
                "tool_name": tool_name,
                "outcome": outcome,
                "duration_bucket": self._duration_bucket(duration_ms),
                "installation_id_hash": self._installation_id_hash(),
            }
        )
        if error is not None:
            error_category = self._classify_error(error)
            payload["error_category"] = error_category
            if error_category == "validation":
                event_name = f"tool_error.validation.{tool_name}"
            else:
                event_name = f"tool_error.other.{tool_name}"
        self._schedule_event(event_name, payload)

    async def drain(self) -> None:
        """Test helper: wait for scheduled telemetry tasks to finish."""
        if not self._pending_tasks:
            return
        await asyncio.gather(*tuple(self._pending_tasks), return_exceptions=True)

    def _runtime_context(self, *, deployment_method: DeploymentMethod | None = None) -> dict[str, str]:
        endpoint_host_hash = self._hash_identifier(self._endpoint_host())
        project_hash = self._hash_identifier(str(settings.ALLURE_PROJECT_ID)) if settings.ALLURE_PROJECT_ID else None
        resolved_deployment_method = deployment_method or self._detect_deployment_method()
        return {
            "server_version": self._server_version,
            "python_version": platform.python_version(),
            "platform": f"{platform.system().lower()}-{platform.machine().lower()}",
            "mcp_mode": self._mcp_mode,
            "deployment_method": resolved_deployment_method,
            "project_id_hash": project_hash or "unknown",
            "endpoint_host_hash": endpoint_host_hash or "unknown",
        }

    def _installation_id_hash(self) -> str:
        endpoint_host = self._endpoint_host() or "unknown-host"
        project_id = str(settings.ALLURE_PROJECT_ID) if settings.ALLURE_PROJECT_ID else "unknown-project"
        return self._hash_identifier(f"{endpoint_host}:{project_id}") or "unknown"

    def _endpoint_host(self) -> str | None:
        parsed = urlparse(settings.ALLURE_ENDPOINT)
        return parsed.hostname

    @staticmethod
    def _resolve_umami_base_url(base_url: str) -> str:
        cleaned = base_url.strip().rstrip("/")
        if cleaned.endswith("/api/send"):
            return cleaned[: -len("/api/send")]
        if cleaned.endswith("/api"):
            return cleaned[: -len("/api")]
        return cleaned

    def _configure_umami_client(self) -> bool:
        try:
            umami.set_url_base(self._umami_base_url)
            umami.set_hostname(self._hostname)
            if self._website_id:
                umami.set_website_id(self._website_id)
            return True
        except Exception as exc:
            logger.warning(
                "Telemetry disabled due to invalid Umami configuration",
                extra={"context": {"error": exc.__class__.__name__}},
            )
            return False

    @staticmethod
    def _resolve_enabled(explicit_enabled: bool | None) -> bool:
        if explicit_enabled is not None:
            return explicit_enabled
        if settings.TELEMETRY_ENABLED is not None:
            return settings.TELEMETRY_ENABLED
        return telemetry_config.enabled

    @staticmethod
    def _resolve_umami_website_id(explicit_website_id: str | None) -> str | None:
        if explicit_website_id is not None:
            return explicit_website_id
        if settings.TELEMETRY_WEBSITE_ID is not None:
            return settings.TELEMETRY_WEBSITE_ID
        return telemetry_config.umami_website_id

    @staticmethod
    def _resolve_umami_hostname(explicit_hostname: str | None) -> str:
        if explicit_hostname is not None:
            resolved = explicit_hostname
        elif settings.TELEMETRY_HOSTNAME is not None:
            resolved = settings.TELEMETRY_HOSTNAME
        else:
            resolved = telemetry_config.umami_hostname
        return resolved or "lucius-mcp.local"

    def _schedule_event(self, event_name: str, payload: dict[str, str]) -> None:
        if not self._umami_ready:
            logger.debug("Telemetry client is not configured; skipping event", extra={"context": {"event": event_name}})
            return

        if not self._website_id:
            if not self._missing_website_warned:
                logger.warning(
                    "Telemetry is enabled but telemetry_config.umami_website_id is missing; skipping event",
                    extra={"context": {"event": event_name}},
                )
                self._missing_website_warned = True
            else:
                logger.debug(
                    "Telemetry event skipped due to missing telemetry_config.umami_website_id",
                    extra={"context": {"event": event_name}},
                )
            return

        try:
            loop = asyncio.get_running_loop()
        except RuntimeError:
            logger.debug("No running event loop; telemetry event skipped", extra={"context": {"event": event_name}})
            return

        # Intentionally fire-and-forget: telemetry must never block tool execution or startup.
        task = loop.create_task(self._send_event(event_name=event_name, payload=payload))
        self._pending_tasks.add(task)
        task.add_done_callback(self._pending_tasks.discard)

    async def _send_event(self, *, event_name: str, payload: dict[str, str]) -> None:
        if not self._website_id:
            return

        try:
            await umami.new_event_async(
                event_name=event_name,
                hostname=self._hostname,
                url="/",
                website_id=self._website_id,
                title=event_name,
                custom_data=payload,
            )
        except httpx.HTTPStatusError as exc:
            logger.warning(
                "Telemetry emission failed",
                extra={
                    "context": {
                        "event": event_name,
                        "error": exc.__class__.__name__,
                        "status_code": str(exc.response.status_code),
                    }
                },
            )
        except httpx.HTTPError as exc:
            logger.warning(
                "Telemetry emission failed",
                extra={"context": {"event": event_name, "error": exc.__class__.__name__}},
            )
        except Exception as exc:
            logger.warning(
                "Telemetry emission failed",
                extra={"context": {"event": event_name, "error": exc.__class__.__name__}},
            )

    def _detect_deployment_method(self) -> DeploymentMethod:
        """Infer deployment method using process/runtime heuristics.

        Detection order is intentionally strict:
        1. `docker`: container markers (`/.dockerenv`, Kubernetes env hint)
        2. `mcpb`: bundled runtime markers in argv/executable/env
        3. `uvx+pypi`: uvx runtime markers in argv/env
        4. `cli`: installed script entrypoint names
        5. fallback: `plain-code-checkout`
        """
        if Path("/.dockerenv").exists() or os.getenv("KUBERNETES_SERVICE_HOST"):
            return "docker"

        argv_joined = " ".join(sys.argv).lower()
        executable = str(Path(sys.executable)).lower()
        if "mcpb" in argv_joined or "mcpb" in executable or os.getenv("MCPB_BUNDLE_PATH"):
            return "mcpb"

        if "uvx" in argv_joined:
            return "uvx+pypi"

        executable_name = Path(sys.argv[0]).name.lower()
        if executable_name in {"start", "lucius-mcp", "lucius_mcp"}:
            return "cli"

        return "plain-code-checkout"

    def _hash_identifier(self, value: str | None) -> str | None:
        if not value:
            return None
        salted = f"{self._hash_salt}:{value}" if self._hash_salt else value
        return hashlib.sha256(salted.encode("utf-8")).hexdigest()

    @staticmethod
    def _duration_bucket(duration_ms: float) -> str:
        if duration_ms < 100:
            return "<100ms"
        if duration_ms < 500:
            return "100-500ms"
        if duration_ms < 2000:
            return "500ms-2s"
        return ">2s"

    @staticmethod
    def _classify_error(error: Exception) -> TelemetryErrorCategory:
        error_name = error.__class__.__name__.lower()
        if isinstance(error, (ValidationError, ValueError, TypeError)) or "validation" in error_name:
            return "validation"
        if "auth" in error_name or "permission" in error_name or "token" in error_name or "unauthorized" in error_name:
            return "auth"
        if "api" in error_name or "http" in error_name or "request" in error_name or "timeout" in error_name:
            return "api"
        return "unexpected"



from typing import Annotated

from pydantic import Field

from src.client import AllureClient
from src.services.test_case_service import TestCaseService


async def get_test_case_custom_fields(
    test_case_id: Annotated[int, Field(description="The ID of the test case to retrieve custom fields for")],
    project_id: Annotated[int | None, Field(description="Optional override for the default Project ID.")] = None,
) -> str:
    """Retrieve custom field values for a specific test case.

    Args:
        test_case_id: The ID of the test case.
        project_id: Optional project ID override.

    Returns:
        A dictionary where keys are custom field names and values are
        either a single string value or a list of string values.
    """
    async with AllureClient.from_env(project=project_id) as client:
        service = TestCaseService(client=client)
        cf_values = await service.get_test_case_custom_fields_values(test_case_id)

        if not cf_values:
            return f"Test Case {test_case_id} has no custom field values set."

        lines = [f"Custom Fields for Test Case {test_case_id}:"]
        for name, value in cf_values.items():
            val_str = ", ".join(value) if isinstance(value, list) else str(value)
            lines.append(f"- {name}: {val_str}")
        return "\n".join(lines)
